@@ -21,38 +21,43 @@
 #                                        02111-1307  USA
 #
 #
-import os, sys
+import os
+import sys
 import selinux
 import sepolicy
 from sepolicy import get_os_version, get_conditionals, get_conditionals_format_text
 import argparse
 import gettext
-PROGNAME="policycoreutils"
+PROGNAME = "policycoreutils"
 gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
 gettext.textdomain(PROGNAME)
 try:
     gettext.install(PROGNAME,
                     localedir="/usr/share/locale",
                     unicode=False,
-                    codeset = 'utf-8')
+                    codeset='utf-8')
 except IOError:
     import __builtin__
     __builtin__.__dict__['_'] = unicode
 
 usage = "sepolicy generate [-h] [-n NAME] [-p PATH] ["
-usage_dict = {' --newtype':('-t [TYPES [TYPES ...]]',),' --customize':('-d DOMAIN','-a  ADMIN_DOMAIN',"[ -w WRITEPATHS ]",), ' --admin_user':('[-r TRANSITION_ROLE ]',"[ -w WRITEPATHS ]",), ' --application':('COMMAND',"[ -w WRITEPATHS ]",), ' --cgi':('COMMAND',"[ -w WRITEPATHS ]",), ' --confined_admin':('-a  ADMIN_DOMAIN',"[ -w WRITEPATHS ]",), ' --dbus':('COMMAND',"[ -w WRITEPATHS ]",), ' --desktop_user':('',"[ -w WRITEPATHS ]",),' --inetd':('COMMAND',"[ -w WRITEPATHS ]",),' --init':('COMMAND',"[ -w WRITEPATHS ]",), ' --sandbox':("[ -w WRITEPATHS ]",), ' --term_user':("[ -w WRITEPATHS ]",), ' --x_user':("[ -w WRITEPATHS ]",)}
+usage_dict = {' --newtype': ('-t [TYPES [TYPES ...]]',), ' --customize': ('-d DOMAIN', '-a  ADMIN_DOMAIN', "[ -w WRITEPATHS ]",), ' --admin_user': ('[-r TRANSITION_ROLE ]', "[ -w WRITEPATHS ]",), ' --application': ('COMMAND', "[ -w WRITEPATHS ]",), ' --cgi': ('COMMAND', "[ -w WRITEPATHS ]",), ' --confined_admin': ('-a  ADMIN_DOMAIN', "[ -w WRITEPATHS ]",), ' --dbus': ('COMMAND', "[ -w WRITEPATHS ]",), ' --desktop_user': ('', "[ -w WRITEPATHS ]",), ' --inetd': ('COMMAND', "[ -w WRITEPATHS ]",), ' --init': ('COMMAND', "[ -w WRITEPATHS ]",), ' --sandbox': ("[ -w WRITEPATHS ]",), ' --term_user': ("[ -w WRITEPATHS ]",), ' --x_user': ("[ -w WRITEPATHS ]",)}
+
 
 class CheckPath(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         if not os.path.exists(values):
-                raise ValueError("%s does not exist" % values)
+            raise ValueError("%s does not exist" % values)
         setattr(namespace, self.dest, values)
 
+
 class CheckType(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         domains = sepolicy.get_all_domains()
 
-        if isinstance(values,str):
+        if isinstance(values, str):
             setattr(namespace, self.dest, values)
         else:
             newval = getattr(namespace, self.dest)
@@ -63,14 +68,16 @@ class CheckType(argparse.Action):
                 newval.append(v)
             setattr(namespace, self.dest, newval)
 
+
 class CheckBoolean(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         booleans = sepolicy.get_all_booleans()
         newval = getattr(namespace, self.dest)
         if not newval:
             newval = []
 
-        if isinstance(values,str):
+        if isinstance(values, str):
             v = selinux.selinux_boolean_sub(values)
             if v not in booleans:
                 raise ValueError("%s must be an SELinux process domain:\nValid domains: %s" % (v, ", ".join(booleans)))
@@ -84,11 +91,13 @@ class CheckBoolean(argparse.Action):
                 newval.append(v)
             setattr(namespace, self.dest, newval)
 
+
 class CheckDomain(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         domains = sepolicy.get_all_domains()
 
-        if isinstance(values,str):
+        if isinstance(values, str):
             if values not in domains:
                 raise ValueError("%s must be an SELinux process domain:\nValid domains: %s" % (values, ", ".join(domains)))
             setattr(namespace, self.dest, values)
@@ -104,17 +113,22 @@ class CheckDomain(argparse.Action):
             setattr(namespace, self.dest, newval)
 
 all_classes = None
+
+
 class CheckClass(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         global all_classes
         if not all_classes:
-                all_classes = map(lambda x: x['name'], sepolicy.info(sepolicy.TCLASS))
+            all_classes = map(lambda x: x['name'], sepolicy.info(sepolicy.TCLASS))
         if values not in all_classes:
             raise ValueError("%s must be an SELinux class:\nValid classes: %s" % (values, ", ".join(all_classes)))
 
         setattr(namespace, self.dest, values)
 
+
 class CheckAdmin(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         from sepolicy.interface import get_admin
         newval = getattr(namespace, self.dest)
@@ -126,7 +140,9 @@ class CheckAdmin(argparse.Action):
         newval.append(values)
         setattr(namespace, self.dest, newval)
 
+
 class CheckPort(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         newval = getattr(namespace, self.dest)
         if not newval:
@@ -137,7 +153,9 @@ class CheckPort(argparse.Action):
             newval.append(v)
         setattr(namespace, self.dest, newval)
 
+
 class CheckPortType(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         port_types = sepolicy.get_all_port_types()
         newval = getattr(namespace, self.dest)
@@ -149,13 +167,17 @@ class CheckPortType(argparse.Action):
             newval.append(v)
         setattr(namespace, self.dest, values)
 
+
 class LoadPolicy(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         import sepolicy
         sepolicy.policy(values)
         setattr(namespace, self.dest, values)
 
+
 class CheckPolicyType(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
         from sepolicy.generate import get_poltype_desc, poltype
         if values not in poltype.keys():
@@ -163,31 +185,37 @@ class CheckPolicyType(argparse.Action):
             newval.append(v)
         setattr(namespace, self.dest, values)
 
+
 class CheckUser(argparse.Action):
+
     def __call__(self, parser, namespace, value, option_string=None):
         newval = getattr(namespace, self.dest)
         if not newval:
             newval = []
         users = sepolicy.get_all_users()
         if value not in users:
-                raise ValueError("%s must be an SELinux user:\nValid users: %s" % (value, ", ".join(users)))
+            raise ValueError("%s must be an SELinux user:\nValid users: %s" % (value, ", ".join(users)))
         newval.append(value)
         setattr(namespace, self.dest, newval)
 
+
 class CheckRole(argparse.Action):
+
     def __call__(self, parser, namespace, value, option_string=None):
         newval = getattr(namespace, self.dest)
         if not newval:
             newval = []
         roles = sepolicy.get_all_roles()
         if value not in roles:
-                raise ValueError("%s must be an SELinux role:\nValid roles: %s" % (value, ", ".join(roles)))
+            raise ValueError("%s must be an SELinux role:\nValid roles: %s" % (value, ", ".join(roles)))
         newval.append(value[:-2])
         setattr(namespace, self.dest, newval)
 
+
 class InterfaceInfo(argparse.Action):
+
     def __call__(self, parser, namespace, values, option_string=None):
-	from sepolicy.interface import get_interface_dict
+        from sepolicy.interface import get_interface_dict
         interface_dict = get_interface_dict()
         for v in values:
             if v not in interface_dict.keys():
@@ -195,19 +223,21 @@ class InterfaceInfo(argparse.Action):
 
         setattr(namespace, self.dest, values)
 
-def generate_custom_usage(usage_text,usage_dict):
+
+def generate_custom_usage(usage_text, usage_dict):
     sorted_keys = []
     for i in usage_dict.keys():
         sorted_keys.append(i)
     sorted_keys.sort()
     for k in sorted_keys:
-        usage_text += "%s %s |" % (k,(" ".join(usage_dict[k])))
+        usage_text += "%s %s |" % (k, (" ".join(usage_dict[k])))
     usage_text = usage_text[:-1] + "]"
     usage_text = _(usage_text)
 
     return usage_text
 
-def numcmp(val1,val2):
+
+def numcmp(val1, val2):
     try:
         v1 = int(val1.split(",")[0].split("-")[0])
         v2 = int(val2.split(",")[0].split("-")[0])
@@ -218,28 +248,30 @@ def numcmp(val1,val2):
         if v1 < v2:
             return -1
     except:
-        return cmp(val1,val2)
+        return cmp(val1, val2)
+
 
 def _print_net(src, protocol, perm):
     import sepolicy.network
     portdict = sepolicy.network.get_network_connect(src, protocol, perm)
     if len(portdict) > 0:
-        bold_start="\033[1m"
-        bold_end="\033[0;0m"
-        print "\n"+bold_start+"%s: %s %s" % (src, protocol, perm) + bold_end
-        port_strings=[]
-        boolean_text=""
+        bold_start = "\033[1m"
+        bold_end = "\033[0;0m"
+        print "\n" + bold_start + "%s: %s %s" % (src, protocol, perm) + bold_end
+        port_strings = []
+        boolean_text = ""
         for p in portdict:
             for t, recs in portdict[p]:
-                cond=get_conditionals(src,t,"%s_socket" % protocol, [perm])
+                cond = get_conditionals(src, t, "%s_socket" % protocol, [perm])
                 if cond:
-                    boolean_text=get_conditionals_format_text(cond)
+                    boolean_text = get_conditionals_format_text(cond)
                     port_strings.append("%s (%s) %s" % (", ".join(recs), t, boolean_text))
                 else:
                     port_strings.append("%s (%s)" % (", ".join(recs), t))
         port_strings.sort(numcmp)
         for p in port_strings:
-                print "\t" + p
+            print "\t" + p
+
 
 def network(args):
     portrecs, portrecsbynum = sepolicy.gen_port_dict()
@@ -268,10 +300,10 @@ def network(args):
                 print "Undefined port type"
 
     for t in args.type:
-        if (t,'tcp') in portrecs.keys():
-            print "%s: tcp: %s" % (t, ",".join(portrecs[t,'tcp']))
-        if (t,'udp') in portrecs.keys():
-            print "%s: udp: %s" % (t, ",".join(portrecs[t,'udp']))
+        if (t, 'tcp') in portrecs.keys():
+            print "%s: tcp: %s" % (t, ",".join(portrecs[t, 'tcp']))
+        if (t, 'udp') in portrecs.keys():
+            print "%s: udp: %s" % (t, ",".join(portrecs[t, 'udp']))
 
     for a in args.applications:
         d = sepolicy.get_init_transtype(a)
@@ -283,6 +315,7 @@ def network(args):
         for net in ("tcp", "udp"):
             _print_net(d, net, "name_bind")
 
+
 def gui_run(args):
     try:
         import sepolicy.gui
@@ -291,16 +324,18 @@ def gui_run(args):
     except ImportError:
         raise ValueError(_("You need to install policycoreutils-gui package to use the gui option"))
 
+
 def gen_gui_args(parser):
     gui = parser.add_parser("gui",
                             help=_('Graphical User Interface for SELinux Policy'))
     gui.add_argument("-d", "--domain", default=None,
-                       action=CheckDomain,
-                       help=_("Domain name(s) of man pages to be created"))
+                     action=CheckDomain,
+                     help=_("Domain name(s) of man pages to be created"))
     gui.add_argument("-t", "--test", default=False, action="store_true",
-                       help=argparse.SUPPRESS)
+                     help=argparse.SUPPRESS)
     gui.set_defaults(func=gui_run)
 
+
 def manpage(args):
     from sepolicy.manpage import ManPage, HTMLManPages, manpage_domains, manpage_roles, gen_domains
 
@@ -316,12 +351,13 @@ def manpage(args):
         test_domains = args.domain
 
     for domain in test_domains:
-        m = ManPage(domain, path, args.root,args.source_files, args.web)
+        m = ManPage(domain, path, args.root, args.source_files, args.web)
         print m.get_man_page_path()
 
     if args.web:
         HTMLManPages(manpage_roles, manpage_domains, path, args.os)
 
+
 def gen_manpage_args(parser):
     man = parser.add_parser("manpage",
                             help=_('Generate SELinux man pages'))
@@ -345,37 +381,40 @@ def gen_manpage_args(parser):
                        help=_("Domain name(s) of man pages to be created"))
     man.set_defaults(func=manpage)
 
-def gen_network_args(parser):
-        net = parser.add_parser("network",
-                                   help=_('Query SELinux policy network information'))
 
-        group = net.add_mutually_exclusive_group(required=True)
-        group.add_argument("-l", "--list", dest="list_ports",
-                           action="store_true",
-                            help=_("list all SELinux port types"))
-        group.add_argument("-p", "--port", dest="port", default=[],
-                            action=CheckPort, nargs="+", type=int,
-                            help=_("show SELinux type related to the port"))
-        group.add_argument("-t", "--type", dest="type", default=[],
-                            action=CheckPortType,nargs="+",
-                            help=_("Show ports defined for this SELinux type"))
-        group.add_argument("-d", "--domain", dest="domain", default=[],
-                            action=CheckDomain, nargs="+",
-                            help=_("show ports to which this domain can bind and/or connect"))
-        group.add_argument("-a", "--application", dest="applications", default=[],
-                           nargs="+",
-                           help=_("show ports to which this application can bind and/or connect"))
-        net.set_defaults(func=network)
+def gen_network_args(parser):
+    net = parser.add_parser("network",
+                            help=_('Query SELinux policy network information'))
+
+    group = net.add_mutually_exclusive_group(required=True)
+    group.add_argument("-l", "--list", dest="list_ports",
+                       action="store_true",
+                       help=_("list all SELinux port types"))
+    group.add_argument("-p", "--port", dest="port", default=[],
+                       action=CheckPort, nargs="+", type=int,
+                       help=_("show SELinux type related to the port"))
+    group.add_argument("-t", "--type", dest="type", default=[],
+                       action=CheckPortType, nargs="+",
+                       help=_("Show ports defined for this SELinux type"))
+    group.add_argument("-d", "--domain", dest="domain", default=[],
+                       action=CheckDomain, nargs="+",
+                       help=_("show ports to which this domain can bind and/or connect"))
+    group.add_argument("-a", "--application", dest="applications", default=[],
+                       nargs="+",
+                       help=_("show ports to which this application can bind and/or connect"))
+    net.set_defaults(func=network)
+
 
 def communicate(args):
-        from sepolicy.communicate import get_types
+    from sepolicy.communicate import get_types
 
-        writable = get_types(args.source, args.tclass, args.sourceaccess.split(","))
-        readable = get_types(args.target, args.tclass, args.targetaccess.split(","))
-        out = list(set(writable) & set(readable))
+    writable = get_types(args.source, args.tclass, args.sourceaccess.split(","))
+    readable = get_types(args.target, args.tclass, args.targetaccess.split(","))
+    out = list(set(writable) & set(readable))
+
+    for t in out:
+        print t
 
-        for t in out:
-            print t
 
 def gen_communicate_args(parser):
     comm = parser.add_parser("communicate",
@@ -389,10 +428,11 @@ def gen_communicate_args(parser):
     comm.add_argument("-c", "--class", required=False, dest="tclass",
                       action=CheckClass,
                       default="file", help="class to use for communications, Default 'file'")
-    comm.add_argument("-S", "--sourceaccess", required=False, dest="sourceaccess",  default="open,write", help="comma separate list of permissions for the source type to use, Default 'open,write'")
-    comm.add_argument("-T", "--targetaccess", required=False, dest="targetaccess",  default="open,read", help="comma separated list of permissions for the target type to use, Default 'open,read'")
+    comm.add_argument("-S", "--sourceaccess", required=False, dest="sourceaccess", default="open,write", help="comma separate list of permissions for the source type to use, Default 'open,write'")
+    comm.add_argument("-T", "--targetaccess", required=False, dest="targetaccess", default="open,read", help="comma separated list of permissions for the target type to use, Default 'open,read'")
     comm.set_defaults(func=communicate)
 
+
 def booleans(args):
     from sepolicy import boolean_desc
     if args.all:
@@ -402,6 +442,7 @@ def booleans(args):
     for b in args.booleans:
         print "%s=_(\"%s\")" % (b, boolean_desc(b))
 
+
 def gen_booleans_args(parser):
     bools = parser.add_parser("booleans",
                               help=_('query SELinux Policy to see description of booleans'))
@@ -414,11 +455,13 @@ def gen_booleans_args(parser):
                        help=_("boolean to get description"))
     bools.set_defaults(func=booleans)
 
+
 def transition(args):
     from sepolicy.transition import setrans
     mytrans = setrans(args.source, args.target)
     mytrans.output()
 
+
 def gen_transition_args(parser):
     trans = parser.add_parser("transition",
                               help=_('query SELinux Policy to see how a source process domain can transition to the target process domain'))
@@ -430,6 +473,7 @@ def gen_transition_args(parser):
                        help=_("target process domain"))
     trans.set_defaults(func=transition)
 
+
 def print_interfaces(interfaces, args, append=""):
     from sepolicy.interface import get_interface_format_text, interface_compile_test
     for i in interfaces:
@@ -446,6 +490,7 @@ def print_interfaces(interfaces, args, append=""):
         else:
             print i
 
+
 def interface(args):
     from sepolicy.interface import get_admin, get_user, get_interface_dict, get_all_interfaces
     if args.list_admin:
@@ -455,13 +500,14 @@ def interface(args):
     if args.list:
         print_interfaces(get_all_interfaces(args.file), args)
     if args.interfaces:
-            print_interfaces(args.interfaces, args)
+        print_interfaces(args.interfaces, args)
+
 
 def generate(args):
     from sepolicy.generate import policy, AUSER, RUSER, EUSER, USERS, SANDBOX, APPLICATIONS, NEWTYPE
     cmd = None
 # numbers present POLTYPE defined in sepolicy.generate
-    conflict_args = {'TYPES':(NEWTYPE,), 'DOMAIN':(EUSER,), 'ADMIN_DOMAIN':(AUSER, RUSER, EUSER,)}
+    conflict_args = {'TYPES': (NEWTYPE,), 'DOMAIN': (EUSER,), 'ADMIN_DOMAIN': (AUSER, RUSER, EUSER,)}
     error_text = ""
 
     if args.policytype is None:
@@ -477,7 +523,7 @@ def generate(args):
             raise ValueError(_("Command required for this type of policy"))
         cmd = os.path.realpath(args.command)
         if not args.name:
-            args.name = os.path.basename(cmd).replace("-","_")
+            args.name = os.path.basename(cmd).replace("-", "_")
 
     mypolicy = policy(args.name, args.policytype)
     if cmd:
@@ -498,7 +544,7 @@ def generate(args):
 
     if len(args.writepaths) > 0 and args.policytype == NEWTYPE:
 
-            raise ValueError(_("-w option can not be used with the --newtype option"))
+        raise ValueError(_("-w option can not be used with the --newtype option"))
 
     for p in args.writepaths:
         if os.path.isdir(p):
@@ -516,6 +562,7 @@ def generate(args):
         mypolicy.gen_symbols()
     print mypolicy.generate(args.path)
 
+
 def gen_interface_args(parser):
     itf = parser.add_parser("interface",
                             help=_('List SELinux Policy interfaces'))
@@ -528,12 +575,12 @@ def gen_interface_args(parser):
     itf.add_argument("-f", "--file", dest="file",
                      help="Interface file")
     group = itf.add_mutually_exclusive_group(required=True)
-    group.add_argument("-a", "--list_admin", dest="list_admin",action="store_true",                       default=False,
+    group.add_argument("-a", "--list_admin", dest="list_admin", action="store_true", default=False,
                        help="List all domains with admin interface - DOMAIN_admin()")
-    group.add_argument("-u", "--list_user", dest="list_user",action="store_true",
+    group.add_argument("-u", "--list_user", dest="list_user", action="store_true",
                        default=False,
                        help="List all domains with SELinux user role interface - DOMAIN_role()")
-    group.add_argument("-l", "--list", dest="list",action="store_true",
+    group.add_argument("-l", "--list", dest="list", action="store_true",
                        default=False,
                        help="List all interfaces")
     group.add_argument("-i", "--interfaces", nargs="+", dest="interfaces",
@@ -541,23 +588,24 @@ def gen_interface_args(parser):
                        help=_("Enter interface names, you wish to query"))
     itf.set_defaults(func=interface)
 
+
 def gen_generate_args(parser):
     from sepolicy.generate import DAEMON, get_poltype_desc, poltype, DAEMON, DBUS, INETD, CGI, SANDBOX, USER, EUSER, TUSER, XUSER, LUSER, AUSER, RUSER, NEWTYPE
 
     generate_usage = generate_custom_usage(usage, usage_dict)
 
-    pol = parser.add_parser("generate", usage = generate_usage,
+    pol = parser.add_parser("generate", usage=generate_usage,
                             help=_('Generate SELinux Policy module template'))
     pol.add_argument("-d", "--domain", dest="domain", default=[],
                      action=CheckDomain, nargs="*",
                      help=_("Enter domain type which you will be extending"))
     pol.add_argument("-u", "--user", dest="user", default=[],
-                     action=CheckUser, 
+                     action=CheckUser,
                      help=_("Enter SELinux user(s) which will transition to this domain"))
     pol.add_argument("-r", "--role", dest="role", default=[],
                      action=CheckRole,
                      help=_("Enter SELinux role(s) to which the administror domain will transition"))
-    pol.add_argument("-a", "--admin", dest="admin_domain",default=[],
+    pol.add_argument("-a", "--admin", dest="admin_domain", default=[],
                      action=CheckAdmin,
                      help=_("Enter domain(s) which this confined admin will administrate"))
     pol.add_argument("-n", "--name", dest="name",
@@ -566,29 +614,29 @@ def gen_generate_args(parser):
     pol.add_argument("-T", "--test", dest="test", default=False, action="store_true",
                      help=argparse.SUPPRESS)
     pol.add_argument("-t", "--type", dest="types", default=[], nargs="*",
-                     action=CheckType, 
+                     action=CheckType,
                      help="Enter type(s) for which you will generate new definition and rule(s)")
     pol.add_argument("-p", "--path", dest="path", default=os.getcwd(),
                      help=_("path in which the generated policy files will be stored"))
-    pol.add_argument("-w", "--writepath", dest="writepaths", nargs="*", default = [],
+    pol.add_argument("-w", "--writepath", dest="writepaths", nargs="*", default=[],
                      help=_("path to which the confined processes will need to write"))
     cmdtype = pol.add_argument_group(_("Policy types which require a command"))
     cmdgroup = cmdtype.add_mutually_exclusive_group(required=False)
     cmdgroup.add_argument("--application", dest="policytype", const=USER,
-                       action="store_const",
-                       help=_("Generate '%s' policy") % poltype[USER])
+                          action="store_const",
+                          help=_("Generate '%s' policy") % poltype[USER])
     cmdgroup.add_argument("--cgi", dest="policytype", const=CGI,
-                       action="store_const",
-                       help=_("Generate '%s' policy") % poltype[CGI])
+                          action="store_const",
+                          help=_("Generate '%s' policy") % poltype[CGI])
     cmdgroup.add_argument("--dbus", dest="policytype", const=DBUS,
-                       action="store_const",
-                       help=_("Generate '%s' policy") % poltype[DBUS])
+                          action="store_const",
+                          help=_("Generate '%s' policy") % poltype[DBUS])
     cmdgroup.add_argument("--inetd", dest="policytype", const=INETD,
-                       action="store_const",
-                       help=_("Generate '%s' policy") % poltype[INETD])
+                          action="store_const",
+                          help=_("Generate '%s' policy") % poltype[INETD])
     cmdgroup.add_argument("--init", dest="policytype", const=DAEMON,
-                       action="store_const", default=DAEMON,
-                       help=_("Generate '%s' policy") % poltype[DAEMON])
+                          action="store_const", default=DAEMON,
+                          help=_("Generate '%s' policy") % poltype[DAEMON])
 
     type = pol.add_argument_group("Policy types which do not require a command")
     group = type.add_mutually_exclusive_group(required=False)
@@ -616,7 +664,7 @@ def gen_generate_args(parser):
     group.add_argument("--x_user", dest="policytype", const=XUSER,
                        action="store_const",
                        help=_("Generate '%s' policy") % poltype[XUSER])
-    pol.add_argument("command",nargs="?", default=None,
+    pol.add_argument("command", nargs="?", default=None,
                      help=_("executable to confine"))
     pol.set_defaults(func=generate)
 
@@ -637,15 +685,18 @@ if __name__ == '__main__':
 
     try:
         if os.path.basename(sys.argv[0]) == "sepolgen":
-            args = parser.parse_args([ "generate" ] + sys.argv[1:])
+            parser_args = [ "generate" ] + sys.argv[1:]
+        elif len(sys.argv) > 1:
+            parser_args = sys.argv[1:]
         else:
-            args = parser.parse_args()
+            parser_args = ["-h"]
+        args = parser.parse_args(args=parser_args)
         args.func(args)
         sys.exit(0)
-    except ValueError,e:
+    except ValueError, e:
         sys.stderr.write("%s: %s\n" % (e.__class__.__name__, str(e)))
         sys.exit(1)
-    except IOError,e:
+    except IOError, e:
         sys.stderr.write("%s: %s\n" % (e.__class__.__name__, str(e)))
         sys.exit(1)
     except KeyboardInterrupt:
